@@ -29,7 +29,6 @@ through the code to commit the whitespace formatting.
 import copy
 
 from yapf.yapflib import format_token
-from yapf.yapflib import pytree_utils
 from yapf.yapflib import split_penalty
 from yapf.yapflib import style
 from yapf.yapflib import unwrapped_line
@@ -80,6 +79,7 @@ class FormatDecisionState(object):
     self.first_indent = first_indent
     self.newline = False
     self.previous = None
+    self.column_limit = style.Get('COLUMN_LIMIT')
     self._MoveStateToNextToken()
 
   def Clone(self):
@@ -124,7 +124,6 @@ class FormatDecisionState(object):
     """Returns True if the line must split before the next token."""
     current = self.next_token
     previous = current.previous_token
-    column_limit = style.Get('COLUMN_LIMIT')
 
     if current.must_break_before:
       return True
@@ -134,14 +133,17 @@ class FormatDecisionState(object):
       bracket = current if current.ClosesScope() else previous
       if format_token.Subtype.SUBSCRIPT_BRACKET not in bracket.subtypes:
         if bracket.OpensScope():
+          if style.Get('COALESCE_BRACKETS'):
+            if current.OpensScope():
+              return False
+
           if (not _IsLastScopeInLine(bracket) or
               unwrapped_line.IsSurroundedByBrackets(bracket)):
             last_token = bracket.matching_bracket
           else:
             last_token = _LastTokenInLine(bracket.matching_bracket)
 
-          length = last_token.total_length - bracket.total_length
-          if length + self.column >= column_limit:
+          if not self._FitsOnLine(bracket, last_token):
             self.stack[-1].split_before_closing_bracket = True
             return True
 
@@ -165,69 +167,92 @@ class FormatDecisionState(object):
     if format_token.Subtype.DICT_SET_GENERATOR in current.subtypes:
       return True
 
-    if (style.Get('SPLIT_BEFORE_NAMED_ASSIGNS') and
+    if (style.Get('SPLIT_BEFORE_NAMED_ASSIGNS') and not current.is_comment and
         format_token.Subtype.DEFAULT_OR_NAMED_ASSIGN_ARG_LIST in
         current.subtypes):
-      if (previous.value not in {'(', '=', '*', '**'} and
-          current.value not in '=,)'):
-        opening = _GetOpeningParen(current)
-        if opening:
-          arglist_length = (opening.matching_bracket.total_length -
-                            opening.total_length + self.column)
-          return arglist_length > column_limit
+      if (previous.value not in {'=', ':', '*', '**'} and
+          current.value not in ':=,)'):
+        # If we're going to split the lines because of named arguments, then we
+        # want to split after the opening bracket as well. But not when this is
+        # part of function definition.
+        if not _IsFunctionDefinition(previous):
+          # Make sure we don't split after the opening bracket if the
+          # continuation indent is greater than the opening bracket:
+          #
+          #  a(
+          #      b=1,
+          #      c=2)
+          indent_amt = self.stack[-1].indent * style.Get('INDENT_WIDTH')
+          pptoken = previous.previous_token
+          opening_column = len(pptoken.value) if pptoken else 0 - indent_amt - 1
+          if previous.value == '(':
+            return opening_column >= style.Get('CONTINUATION_INDENT_WIDTH')
+          opening = _GetOpeningBracket(current)
+          if opening:
+            arglist_length = (opening.matching_bracket.total_length -
+                              opening.total_length + self.stack[-1].indent)
+            return arglist_length > self.column_limit
 
     if style.Get('SPLIT_ARGUMENTS_WHEN_COMMA_TERMINATED'):
       # Split before arguments in a function call or definition if the
       # arguments are terminated by a comma.
-      opening = _GetOpeningParen(current)
+      opening = _GetOpeningBracket(current)
       if opening and opening.previous_token and opening.previous_token.is_name:
         if previous.value in '(,':
           if opening.matching_bracket.previous_token.value == ',':
             return True
 
+    if (format_token.Subtype.DICTIONARY_VALUE in current.subtypes or
+        (previous.is_pseudo_paren and previous.value == '(')):
+      if not current.OpensScope():
+        opening = _GetOpeningBracket(current)
+        if previous.is_pseudo_paren:
+          # Split before the dictionary value if we can't fit the whole
+          # dictionary on one line.
+          if not self._AllDictElementsFitOnOneLine(opening):
+            return True
+
     if (previous.value in '{[' and current.lineno != previous.lineno and
         format_token.Subtype.SUBSCRIPT_BRACKET not in previous.subtypes):
       # Retain the split after the container opening.
       return True
 
-    if (previous.value == ':' and _IsDictionaryValue(current) and
-        current.lineno != previous.lineno):
-      # Retain the split between the dictionary key and value.
-      return True
+    if previous.value == '{':
+      closing = previous.matching_bracket
+      if (not self._FitsOnLine(previous, closing) and
+          closing.previous_token.value == ','):
+        self.stack[-1].split_before_closing_bracket = True
+        return True
 
     if (format_token.Subtype.COMP_FOR in current.subtypes and
         format_token.Subtype.COMP_FOR not in previous.subtypes):
       # Split at the beginning of a list comprehension.
       length = _GetLengthOfSubtype(current, format_token.Subtype.COMP_FOR,
                                    format_token.Subtype.COMP_IF)
-      if length + self.column > column_limit:
+      if length + self.column > self.column_limit:
         return True
 
     if (format_token.Subtype.COMP_IF in current.subtypes and
         format_token.Subtype.COMP_IF not in previous.subtypes):
       # Split at the beginning of an if expression.
       length = _GetLengthOfSubtype(current, format_token.Subtype.COMP_IF)
-      if length + self.column > column_limit:
+      if length + self.column > self.column_limit:
         return True
 
     previous_previous_token = previous.previous_token
     if (current.is_name and previous_previous_token and
         previous_previous_token.is_name and previous.value == '('):
-      sibling = previous.node.next_sibling
-      if pytree_utils.NodeName(sibling) == 'arglist':
-        arglist = previous.node.next_sibling
-        if len(arglist.children) > 2:
-          if _IsFunctionCallWithArguments(current):
-            # There is a function call, with more than 1 argument, where
-            # the first argument is itself a function call with arguments.
-            # In this specific case, if we split after the first argument's
-            # opening '(', then the formatting will look bad for the rest
-            # of the arguments. Instead, enforce a split before that
-            # argument to keep things looking good.
-            return True
-          elif (current.OpensScope() and
-                current.matching_bracket.total_length + self.column >
-                column_limit):
+      if not self._FitsOnLine(previous, previous.matching_bracket):
+        if _IsFunctionCallWithArguments(current):
+          # There is a function call, with more than 1 argument, where
+          # the first argument is itself a function call with arguments.
+          # In this specific case, if we split after the first argument's
+          # opening '(', then the formatting will look bad for the rest
+          # of the arguments. Instead, enforce a split before that
+          # argument to keep things looking good.
+          return True
+        elif current.OpensScope():
+          if not self._FitsOnLine(current, current.matching_bracket):
             # There is a data literal that will need to be split and could mess
             # up the formatting.
             return True
@@ -244,6 +269,27 @@ class FormatDecisionState(object):
       # original comments were on a separate line.
       return True
 
+    opening = _GetOpeningBracket(current)
+    if (opening and opening.value == '(' and current.is_name and
+        previous.value == ','):
+      # If we have a function call within an argument list and it won't fit on
+      # the remaining line, but it will fit on a line by itself, then go ahead
+      # and split before the call.
+      total_len = 0
+      ntoken = current
+      while ntoken:
+        if ntoken.value == '(':
+          total_len = (
+              ntoken.matching_bracket.total_length - current.total_length)
+          break
+        ntoken = ntoken.next_token
+
+      if ntoken:
+        indent_amt = self.stack[-1].indent * style.Get('INDENT_WIDTH')
+        if (total_len + indent_amt < self.column_limit and
+            total_len + self.column >= self.column_limit):
+          return True
+
     return False
 
   def AddTokenToState(self, newline, dry_run, must_split=False):
@@ -438,8 +484,8 @@ class FormatDecisionState(object):
 
     # Calculate the penalty for overflowing the column limit.
     penalty = 0
-    if self.column > style.Get('COLUMN_LIMIT'):
-      excess_characters = self.column - style.Get('COLUMN_LIMIT')
+    if not current.is_pylint_comment and self.column > self.column_limit:
+      excess_characters = self.column - self.column_limit
       penalty += style.Get('SPLIT_PENALTY_EXCESS_CHARACTER') * excess_characters
 
     if is_multiline_string:
@@ -449,6 +495,25 @@ class FormatDecisionState(object):
 
     return penalty
 
+  def _FitsOnLine(self, start, end):
+    """Determines if line between start and end can fit on the current line."""
+    length = end.total_length - start.total_length
+    return length + self.column < self.column_limit
+
+  def _AllDictElementsFitOnOneLine(self, opening):
+    closing = opening.matching_bracket
+    current, previous = opening.next_token, opening
+    start = None
+    while current and current != closing:
+      if format_token.Subtype.DICTIONARY_KEY in current.subtypes:
+        if start and not self._FitsOnLine(start, current.previous_token):
+          return False
+        start = current
+      if current.OpensScope():
+        current = current.matching_bracket
+      current = current.next_token
+    return start and self._FitsOnLine(start, current.previous_token)
+
 
 def _IsFunctionCallWithArguments(token):
   while token:
@@ -461,14 +526,6 @@ def _IsFunctionCallWithArguments(token):
   return False
 
 
-def _IsDictionaryValue(token):
-  while token:
-    if format_token.Subtype.DICTIONARY_VALUE in token.subtypes:
-      return True
-    token = token.previous_token
-  return False
-
-
 def _GetLengthOfSubtype(token, subtype, exclude=None):
   current = token
   while (current.next_token and subtype in current.subtypes and
@@ -477,11 +534,24 @@ def _GetLengthOfSubtype(token, subtype, exclude=None):
   return current.total_length - token.total_length + 1
 
 
-def _GetOpeningParen(current):
+def _GetOpeningBracket(current):
+  if current.matching_bracket and not current.is_pseudo_paren:
+    return current.matching_bracket
+  while current:
+    if current.ClosesScope():
+      current = current.matching_bracket
+    elif current.is_pseudo_paren:
+      current = current.previous_token
+    elif current.OpensScope():
+      return current
+    current = current.previous_token
+  return None
+
   previous = current
-  if previous and previous.matching_bracket:
+  if previous and previous.matching_bracket and not previous.is_pseudo_paren:
     return previous.matching_bracket
-  while previous is not None and previous.matching_bracket is None:
+  previous = previous.previous_token
+  while previous and not previous.matching_bracket:
     previous = previous.previous_token
     if not previous:
       break
@@ -496,6 +566,12 @@ def _LastTokenInLine(current):
   return current
 
 
+def _IsFunctionDefinition(current):
+  prev = current.previous_token
+  return (current.value == '(' and prev and
+          format_token.Subtype.FUNC_DEF in prev.subtypes)
+
+
 def _IsLastScopeInLine(current):
   while current:
     current = current.next_token
@@ -534,7 +610,7 @@ class _ParenState(object):
     state = _ParenState(self.indent, self.last_space)
     state.closing_scope_indent = self.closing_scope_indent
     state.split_before_closing_bracket = self.split_before_closing_bracket
-    self.num_line_splits = self.num_line_splits
+    state.num_line_splits = self.num_line_splits
     return state
 
   def __repr__(self):
