@@ -24,6 +24,7 @@ from yapf.yapflib import style
 # TODO(morbo): Document the annotations in a centralized place. E.g., the
 # README file.
 UNBREAKABLE = 1000 * 1000
+VERY_STRONGLY_CONNECTED = 3000
 DOTTED_NAME = 2500
 STRONGLY_CONNECTED = 2000
 CONTIGUOUS_LIST = 500
@@ -185,8 +186,8 @@ class _TreePenaltyAssigner(pytree_visitor.PyTreeVisitor):
             #   atom tr1() tr2
             # It may be necessary (though undesirable) to split up a previous
             # function call's parentheses to the next line.
-            self._SetUnbreakable(prev_trailer.children[-1])
-          self._SetUnbreakable(cur_trailer.children[0])
+            self._SetStronglyConnected(prev_trailer.children[-1])
+          self._SetStronglyConnected(cur_trailer.children[0])
           prev_trailer_idx = cur_trailer_idx
         else:
           break
@@ -209,7 +210,13 @@ class _TreePenaltyAssigner(pytree_visitor.PyTreeVisitor):
           if last_child_node.value.strip().startswith('#'):
             last_child_node = last_child_node.prev_sibling
           if not style.Get('DEDENT_CLOSING_BRACKETS'):
-            self._SetUnbreakable(last_child_node)
+            if _LastChildNode(last_child_node.prev_sibling).value != ',':
+              if last_child_node.value == ']':
+                self._SetUnbreakable(last_child_node)
+              else:
+                pytree_utils.SetNodeAnnotation(
+                    last_child_node, pytree_utils.Annotation.SPLIT_PENALTY,
+                    VERY_STRONGLY_CONNECTED)
 
           if _FirstChildNode(trailer).lineno == last_child_node.lineno:
             # If the trailer was originally on one line, then try to keep it
@@ -320,8 +327,7 @@ class _TreePenaltyAssigner(pytree_visitor.PyTreeVisitor):
         if node.value in {'(', 'for', 'if'}:
           return
         penalty_annotation = pytree_utils.GetNodeAnnotation(
-            node, pytree_utils.Annotation.SPLIT_PENALTY,
-            default=0)
+            node, pytree_utils.Annotation.SPLIT_PENALTY, default=0)
         if penalty_annotation < penalty:
           pytree_utils.SetNodeAnnotation(node,
                                          pytree_utils.Annotation.SPLIT_PENALTY,
@@ -347,9 +353,8 @@ def _RecAnnotate(tree, annotate_name, annotate_value):
   for child in tree.children:
     _RecAnnotate(child, annotate_name, annotate_value)
   if isinstance(tree, pytree.Leaf):
-    cur_annotate = pytree_utils.GetNodeAnnotation(tree,
-                                                  annotate_name,
-                                                  default=0)
+    cur_annotate = pytree_utils.GetNodeAnnotation(
+        tree, annotate_name, default=0)
     if cur_annotate < annotate_value:
       pytree_utils.SetNodeAnnotation(tree, annotate_name, annotate_value)
 
